precision highp float;
precision highp int;

varying vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform vec2 uInfo;
uniform vec4 uColor;

#define PI 3.141592653

float linear(float t, float b, float c, float d) {
    return mix(b, c, clamp(t/d, 0.0, 1.0));
}
float easeInExpo(float t, float b, float c, float d) {
    float x = clamp(t/d, 0., 1.);
    return mix(b, c, x == 0.0 ? x : pow(2.0, 10.0 * (x - 1.0)));
}
float easeInSine(float t, float b, float c, float d) {
    float x = clamp(t/d, 0., 1.);
    return mix(b, c, 1. - cos(x * PI / 2.));
}

//gimp
float rgb_max(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.r, rgb.b)), mix(rgb.g, rgb.b, step(rgb.g, rgb.b)), step(rgb.r, rgb.g));
}

//gimp
float rgb_min(vec3 rgb) {
    return mix(mix(rgb.r, rgb.b, step(rgb.b, rgb.r)), mix(rgb.g, rgb.b, step(rgb.b, rgb.g)), step(rgb.g, rgb.r));
}

//gimp
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

vec4 greenKey(vec4 foreground) {
    vec4 bg = vec4(foreground.rgb, 0.);
    vec4 fg = foreground.rgba;
    float maxrb = max(fg.r, fg.b);
    float k = clamp((fg.g-maxrb)*5.0, 0.0, 1.0);

    float dg = fg.g;
    fg.g = min(fg.g, maxrb);
    fg += dg - fg.g;

    return vec4(mix(fg, bg, k));
}

vec4 redKey(vec4 foreground) {
    vec4 bg = vec4(foreground.rgb, 0.);
    vec4 fg = foreground.rgba;
    float maxgb = max(fg.g, fg.b);
    float k = clamp((fg.r-maxgb)*25.0*uInfo.r, 0.0, 1.0);

    float dr = fg.r;
    fg.r = mix(fg.r, min(fg.r, maxgb), uInfo.r);
    fg += dr - fg.r;

    return vec4(mix(fg, bg, k));
}

vec4 blueKey(vec4 foreground) {
    vec4 bg = vec4(foreground.rgb, 0.);
    vec4 fg = foreground.rgba;
    float maxrg = max(fg.g, fg.r);
    float k = clamp((fg.b-maxrg)*25.0*uInfo.r, 0.0, 1.0);

    float db = fg.b;
    fg.b = mix(fg.b, min(fg.b, maxrg), uInfo.r);
    fg += db - fg.b;

    return vec4(mix(fg, bg, k));
}

vec4 chroma(vec4 source, vec4 keyColor) {
    float chromaIntensity = uInfo.r;
    if (keyColor.a == 0.0 || chromaIntensity == 0.0) {
        return source;
    }
    vec4 outputColor;
    float colorDistance = length(source.rgb - keyColor.rgb);
    float mul = smoothstep(chromaIntensity * 0.3, chromaIntensity, colorDistance);
    outputColor = source * mul;
    return outputColor;
}

vec3 rgb2ycbcr(vec3 rgb) {
    return mat3(0.299, -0.168736, 0.5,
    0.587, -0.331264, -0.418688,
    0.114, 0.5, -0.081312)*rgb+vec3(0., 0.5, 0.5);
}
vec4 ycbcrKey(vec4 foreground, vec4 keyColor) {
    vec3 foreYCbCr = rgb2ycbcr(foreground.rgb);
    vec3 keyYCbCr = rgb2ycbcr(keyColor.rgb);
    //    float curve = easeInExpo(uInfo.x-0.5, easeInSine(uInfo.x, 0., 0.25, 0.5), sqrt(2.)/0.24, 0.5);
    //    float mask = linear(distance(foreYCbCr.yz, keyYCbCr.yz)-0.24*curve, 0., 1., curve-0.24*curve);
    //    return mask>0.?vec4((foreground.rgb-keyColor.rgb*(1.-mask))/mask, mask):vec4(0.);
    float tolb = uInfo.x*0.46;
    float tola = mix(max(0., tolb-0.01), 0.07, step(0.07, tolb));
    float mask = mix(1.-linear(distance(foreYCbCr.yz, keyYCbCr.yz)-tola, 0., 1., tolb-tola), 0., step(tolb, tola));
    return vec4(max(foreground.r-mask*keyColor.r, 0.),
    max(foreground.g-mask*keyColor.g, 0.),
    max(foreground.b-mask*keyColor.b, 0.),
    foreground.a-mask);

}

void main() {
    vec4 source=texture2D(inputImageTexture, textureCoordinate);
    vec3 hsv = rgb2hsl(uColor.rgb);
    vec4 result = source;

    if ((hsv.x > 0.64 && hsv.x < 0.7) && hsv.y*hsv.z > 0.3) {
        result = ycbcrKey(source, uColor);
    } else if (((hsv.x > 0.97 && hsv.x < 1.) || (hsv.x > 0. && hsv.x < 0.03)) && hsv.y*hsv.z > 0.3) {
        result = ycbcrKey(source, uColor);
    } else if ((hsv.x > 0.24 && hsv.x < 0.42) && hsv.y*hsv.z > 0.2) {
        result = ycbcrKey(source, uColor);
    } else {
        result = chroma(source, uColor);
    }

    float chromaShadow = uInfo.g;
    if (chromaShadow > 0.0) {
        float luminance = dot(source.rgb, vec3(0.2125, 0.7154, 0.0721));
        float increment = chromaShadow * 0.07;
        float overlay = chromaShadow * (1.0 - smoothstep(0.10 + increment, 0.20 + increment, luminance));
        result.a = result.a + overlay - result.a * overlay;
    }

    gl_FragColor = result;
}