precision highp float;

varying vec2 textureCoordinate;
uniform sampler2D inputImageTexture;
uniform float strength;

const float blur = 0.02;

const mat3 yuvMat= mat3(
0.299, - 0.168736, 0.5,
0.587, - 0.331264, -0.418688,
0.114, 0.5, -0.081312);

mat4 contrastMatrix( float contrast )
{
    float t = ( 1.0 - contrast ) / 2.0;

    return mat4( contrast, 0.0, 0.0, 0.0,
    0.0, contrast, 0.0, 0.0,
    0.0, 0, contrast, 0.0,
    t, t, t, 1.0 );

}

//gimp
float rgb_max(vec3 rgb) {
    return max(rgb.r, max(rgb.g, rgb.b));
}

//gimp
float rgb_min(vec3 rgb) {
    return min(rgb.r, min(rgb.g, rgb.b));
}

//gimp
vec3 rgb2hsl(vec3 rgb) {
    float rgbmax = rgb_max(rgb);
    float rgbmin = rgb_min(rgb);
    vec3 hsl = vec3(0.);
    hsl.z = (rgbmax + rgbmin) / 2.;
    if (rgbmax == rgbmin) {
        hsl.y = 0.;
        hsl.x = -1.;
    } else {
        float delta = rgbmax-rgbmin;
        hsl.y = mix(delta/(2.-rgbmax-rgbmin), delta/(rgbmax+rgbmin), step(hsl.z, .5));
        if (delta == 0.) delta = 1.;
        if (rgb.r == rgbmax) hsl.x = (rgb.g-rgb.b)/delta;
        else if (rgb.g == rgbmax) hsl.x = 2. + (rgb.b-rgb.r)/delta;
        else hsl.x = 4.+(rgb.r-rgb.g)/delta;
        hsl.x /= 6.;
        hsl.x = mix(hsl.x+1., hsl.x, step(0., hsl.x));
    }
    return hsl;
}

//gimp
float hsl_value(float n1, float n2, float hue) {
    //    hue = mix(hue-6., mix(hue+6., hue, step(0., hue)), step(hue, 6.));
    if (hue < 0.) {
        hue = hue + 6.;
    } else if (hue > 6.) {
        hue = hue - 6.;
    }

    if (hue > 4.){
        return n1;
    } else if (hue > 3.){
        return n1 + (n2 - n1) * (4. - hue);
    } else if (hue > 1.){
        return n2;
    } else {
        return n1 + (n2 - n1) * hue;
    }
}

//gimp
vec3 hsl2rgb(vec3 hsl) {
    vec3 rgb = vec3(0.);
    if (hsl.y == 0.) {
        rgb.r = hsl.z;
        rgb.g = hsl.z;
        rgb.b = hsl.z;
    } else {
        float m2 = mix(hsl.z + hsl.y - hsl.z*hsl.y, hsl.z*(1.+hsl.y), step(hsl.z, .5));
        float m1 = 2.*hsl.z-m2;
        rgb.r = hsl_value(m1, m2, hsl.x * 6. + 2.);
        rgb.g = hsl_value(m1, m2, hsl.x * 6.);
        rgb.b = hsl_value(m1, m2, hsl.x * 6. - 2.);
    }
    return rgb;
}

float rgb2gray(vec3 c){
    return dot(c, vec3(0.299, 0.587, 0.114));
}

vec3 rgb2ycbcr(vec3 color){
    return ((yuvMat * color) * vec3(219., 224., 224.) + vec3(16., 128., 128.));
}

float linear(float t, float b, float c, float d) {
    return mix(b, c, clamp(t/d, 0.0, 1.0));
}

//gimp adjust: -1~1
float map_hue(float value, float adjust) {
    float res = value + adjust / 2.0;
    return mix(res + 1., mix(res - 1., res, step(res, 1.)), step(0., res));
}

//gimp adjust -1~1
float map_saturation(float value, float adjust) {
    return clamp(value * (adjust + 1.), 0., 1.);
}

//gimp adjust -1~1
float map_lightness(float value, float adjust) {
    return mix(value *(adjust + 1.), value + (adjust* (1.-value)), step(0., adjust));
}

//gimp
vec4 hslproc(in vec4 src, vec3 adjust) {
    vec3 hsl = rgb2hsl(src.rgb);
    hsl.x = map_hue(hsl.x, adjust.x);
    hsl.y = map_saturation(hsl.y, adjust.y);
    hsl.z = map_lightness(hsl.z, adjust.z);
    vec4 dest = src;
    dest.rgb = hsl2rgb(hsl);
    return dest;
}

float pixel_difference(vec3 hsl, vec3 params) {
    float diff = 0.;

    float h = mix(hsl.x + 1., hsl.x, step(params.z-1., hsl.x));

    float center = params.y;
    float m = linear(h-params.x, 0.0, 0.2, blur);
    m = linear(h-params.x-blur, m, 1., center-params.x-blur);
    m = linear(h-center, m, 0.2, params.z-blur-center);
    m = linear(h-params.z+blur, m, 0., blur);
    diff = m;

    return diff;
}

void main(){
    //origin
    vec3 origin = vec3(350./360., 400./360., 450./360.);

    vec4 color = texture2D(inputImageTexture, textureCoordinate);

    vec3 hsl = rgb2hsl(color.rgb);

    float h = mix(hsl.x + 1., hsl.x, step(origin.z - 1., hsl.x));

    float diff = pixel_difference(hsl, origin);

    gl_FragColor = vec4(min(4.0 * hsl.y * diff, 1.0));
}
